﻿using System;
using System.Diagnostics;
using System.Reflection.Metadata;
using System.Text.Json;
using System.Xml.Linq;

namespace HDLNoCGen
{
    static class Program
    {
        static void Main(string[] args)
        {
            string location = "C:/Users/Serha/Desktop/kal";
            string name = "kal";

            string topology = "";
            List<int> generators = new List<int>();
            string queue_type = "pointer";
            string queue_position = "front";
            string arbiter_type = "round_robin";
            string algorithm = "xy";
            int info_width = 8;
            int queue_length = 4;
            bool create_verilog = false;
            Graph graph = null;

            router_options parameters = new router_options();
            string graph_filename = "";

            CancellationTokenSource GeneralPurposeCancellationTokenSource = new CancellationTokenSource();
            CancellationToken GeneralPurposeCancellationToken = GeneralPurposeCancellationTokenSource.Token;

            try
            {
                for (int i = 0; i < args.Length; i++)
                {
                    switch (args[i])
                    {
                        case "-l":
                        case "--location":
                            location = args[++i];
                            break;
                        case "-n":
                        case "--name":
                            name = args[++i];
                            break;
                        case "c":
                            topology = "c";
                            break;
                        case "m":
                            topology = "m";
                            break;
                        case "t":
                            topology = "t";
                            break;
                        case "--queue_type":
                            queue_type = args[++i];
                            break;
                        case "--queue_position":
                            queue_position = args[++i];
                            break;
                        case "--arbiter_type":
                            arbiter_type = args[++i];
                            break;
                        case "--algorithm":
                            algorithm = args[++i];
                            break;
                        case "--info_width":
                            try
                            {
                                info_width = Convert.ToInt32(args[++i]);
                            }
                            catch (Exception e)
                            {
                                Console.WriteLine("--info_width must be a number");
                                Environment.Exit(1);
                            }
                            break;
                        case "--queue_length":
                            try
                            {
                                queue_length = Convert.ToInt32(args[++i]);
                            }
                            catch (Exception e)
                            {
                                Console.WriteLine("--queue_lenght must be a number");
                                Environment.Exit(1);
                            }
                            break;
                        case "-v":
                        case "--verilog":
                            create_verilog = true;
                            break;
                        default:
                            try
                            {
                                generators.Add(Convert.ToInt32(args[i]));
                            }
                            catch (Exception e)
                            {
                                Console.WriteLine($"Argument {args[i]} is invalid");
                            }
                            break;

                    }
                }
            }
            catch (IndexOutOfRangeException e)
            {
                Console.WriteLine("No argument after an option that requires an argument");
                Environment.Exit(1);
            }

            switch (topology)
            {
                case "c":
                    graph = new GraphCirculant();
                    break;
                case "m":
                    graph = new GraphMesh();
                    break;
                case "t":
                    graph = new GraphTorus();
                    break;
                default:
                    Console.WriteLine($"Topology {topology} does not exit");
                    Environment.Exit(1);
                    break;
            }

            graph.InstallGenerators(generators, GeneralPurposeCancellationToken);

            graph_filename += $"{info_width}_bit";

            switch (graph.graphId)
            {
                case GraphType.Circulant:
                    graph_filename += "__circulant";
                    graph_filename += $"_{graph.node_count}_" + String.Join("_", graph.generators.Slice(0, graph.generators.Count - 1).Select(s => $"{s}"));
                    break;
                case GraphType.Mesh:
                    graph_filename += "__mesh";
                    graph_filename += $"_{graph.node_count}_" + String.Join("_", graph.generators.Select(s => $"{s}"));
                    break;
                case GraphType.Torus:
                    graph_filename += "__torus";
                    graph_filename += $"_{graph.node_count}_" + String.Join("_", graph.generators.Select(s => $"{s}"));
                    break;
                default:
                    Console.WriteLine($"Graph type {graph.graphId} was not implemented into this switch statement.");
                    Environment.Exit(1);
                    break;
            }

            switch (queue_type)
            {
                case "index":
                    parameters.queue_type = RouterOptions.Queue_types.Index;
                    graph_filename += $"__indexQueue_{queue_length}_long";
                    break;
                case "pointer":
                    parameters.queue_type = RouterOptions.Queue_types.Pointer;
                    graph_filename += $"__pointerQueue_{queue_length}_long";
                    break;
                case "line":
                    parameters.queue_type = RouterOptions.Queue_types.Line;
                    graph_filename += $"__lineQueue_{queue_length}_long";
                    break;
                default:
                    Console.WriteLine($"Queue type '{queue_length}' does not exist. It should be either 'index', 'pointer' or 'line'.");
                    Environment.Exit(1);
                    break;
            }

            switch (queue_position)
            {
                case "front":
                    parameters.queue_position = RouterOptions.Queue_position.Front;
                    graph_filename += "_front";
                    break;
                case "rear":
                    parameters.queue_position = RouterOptions.Queue_position.Rear;
                    graph_filename += "_rear";
                    break;
                default:
                    Console.WriteLine($"Queue position '{queue_position}' does not exist. It should be either 'front' or 'rear'.");
                    Environment.Exit(1);
                    break;
            }

            switch (arbiter_type)
            {
                case "round_robin":
                    parameters.arbiter_type = RouterOptions.Arbiter_types.Round_Robin;
                    graph_filename += "__arbRoundRobin";
                    break;
                default:
                    Console.WriteLine($"Arbiter type '{queue_position}' does not exist. It should be either 'front' or 'rear'.");
                    Environment.Exit(1);
                    break;
            }

            switch (algorithm)
            {
                case "xy":
                    graph_filename += "__xy";

                    switch (graph.graphId)
                    {
                        case GraphType.Mesh:
                            parameters.algorithm = RouterOptions.Algorithm.XY;
                            break;
                        case GraphType.Torus:
                            parameters.algorithm = RouterOptions.Algorithm.XY;
                            break;
                        default:
                            Console.WriteLine($"Graph type {graph.graphId} was not implemented into this switch statement.");
                            Environment.Exit(1);
                            break;
                    }

                    break;
                case "ga":
                    parameters.algorithm = RouterOptions.Algorithm.GA;
                    graph_filename += "__ga";
                    break;
                default:
                    Console.WriteLine($"Algorithm '{algorithm}' does not exist. It should be either 'xy' or 'ga'.");
                    Environment.Exit(1);
                    break;
            }

            switch (graph.graphId)
            {
                case GraphType.Circulant:
                    parameters.packet_width = 1 + graph.get_bits(graph.node_count) + info_width;
                    break;
                case GraphType.Mesh:
                case GraphType.Torus:
                    parameters.packet_width = 1 + 2 * graph.get_bits(generators[1]) + info_width;
                    break;
            }
            parameters.queue_length = queue_length;
            parameters.info_width = info_width;

            graph.parameters = parameters;
            string json = "";

            switch (graph.graphId)
            {
                case GraphType.Circulant:
                    json = JsonSerializer.Serialize<GraphCirculant>(graph as GraphCirculant, new JsonSerializerOptions { WriteIndented = true });
                    break;
                case GraphType.Mesh:
                    json = JsonSerializer.Serialize<GraphMesh>(graph as GraphMesh, new JsonSerializerOptions { WriteIndented = true });
                    break;
                case GraphType.Torus:
                    json = JsonSerializer.Serialize<GraphTorus>(graph as GraphTorus, new JsonSerializerOptions { WriteIndented = true });
                    break;
            }

            using (StreamWriter sw = new StreamWriter($"{location}/graph_object_serialized.json"))
            {
                sw.Write(json);
            }

            if (!create_verilog)
            {
                Environment.Exit(0);
            }

            Tuple<string, Process, Process> preparation_result = ProjectGenerator.routerPreparation(graph, "NoC_description", location, parameters);
            string verilog_path = preparation_result.Item1;
            Process compile_process = preparation_result.Item2;
            Process simulate_process = preparation_result.Item3;

            graph.createNoC(verilog_path, parameters);

            StreamReader sr = new StreamReader($"{location}\\graph_object_serialized.json");
            string json_deser = sr.ReadToEnd();
            sr.Close();

            GraphDeserializer des_graph_test = JsonSerializer.Deserialize<GraphDeserializer>(json_deser);
            Graph graph_des;
            switch (des_graph_test.graphId)
            {
                case GraphType.Circulant:
                    graph_des = new GraphCirculant(des_graph_test);
                    break;
                case GraphType.Mesh:
                    graph_des = new GraphMesh(des_graph_test);
                    break;
                case GraphType.Torus:
                    graph_des = new GraphTorus(des_graph_test);
                    break;
            }

        }
    }
}