`include "src/router.sv"
`include "inc/queue.svh"
`include "inc/noc.svh"
`include "inc/noc_XY.svh"
`include "inc/router.svh"
`timescale 1ns/1ns


module tb_queueing;


reg clk = 0;
reg[0:`PL-1] inputs[0:`REN-1];
wire[0:`PL-1] outputs[0:`REN-1];
reg signals_in[0:`REN-1];
wire signals_out[0:`REN-1];
reg[`CS-1:0] router_X = `X / 2;
reg[`CS-1:0] router_Y = `Y / 2;

integer i;
integer j;
integer k;
reg[0:`PL-1] packet;
reg[`PL - `CS*2 - 2:0] info;
wire[3:0] arb_shift = DUT.round_robin.shift;

router DUT (
    clk,
    inputs, outputs,
    signals_in, signals_out,
    router_X, router_Y
);

localparam CLK_PERIOD = 10;
always #(CLK_PERIOD/2) clk=~clk;


initial begin
    

    for (k = 1; k < `REN; k = k + 2)
    begin

        for (i = 0; i < `REN; i = i + 1)
        begin
            signals_in[i] = 0;
            inputs[i] = 0;
        end

        for (i = 1; i < `REN + 1; i = i + k)
        begin

            for (j = 0; j < `EN * 2; j = j + 1) // testing with overflow
            begin
                info = ((i % `REN)+1) * (j+1);
                packet = {1'b1, router_X, router_Y, info};
                @(posedge clk)
                begin
                    inputs[(i % `REN)] <= packet;
                end
            end
            @(posedge clk)
            begin
                inputs[(i % `REN)] <= 0;
            end
            
        end

        @(posedge clk)
        begin
            inputs[0] <= 0;
        end
        

        if (k == 1)
            while (arb_shift != 0) @(posedge clk);
        else
            while (arb_shift != 1 + k) @(posedge clk);
        
        for (i = 0; i < `REN; i = i + 1)
        begin
            signals_in[i] = 1;
        end

        for (j = 0; j < `EN; j = j + 1)
        begin

            for (i = 1; i < `REN + 1; i = i + k)
            begin
                info = ((i % `REN)+1) * (j+1);
                packet = {1'b1, router_X, router_Y, info};
                @(posedge clk)
                begin
                    assert (outputs[0] == packet) else $error("Packet %b not to be seen", packet);
                end
            end

        end

    end

    $finish;
end

endmodule