`include "noc/toplevel.sv"
`include "inc/noc.svh"
`include "inc/noc_XY.svh"
`timescale 1ns/1ns

module tb_routing;

reg clk = 0;
reg[0:`PL-1] packet = 0;
reg[`CS-1:0] core_X = 0;
reg[`CS-1:0] core_Y = 0;
reg core_availability_signals_out[0:`Y-1][0:`X-1];
wire core_availability_signals_in[0:`Y-1][0:`X-1];
wire[0:1] core_inputs_2_bits[0:`Y-1][0:`X-1];

reg[0:`PL-1] memory[0:`Y-1][0:`X-1][0:1];

reg[31:0] i = 0;
reg[31:0] j = 0;
reg[31:0] k = 0;
reg[31:0] p = 0;
reg[31:0] t = 0;
reg[31:0] m = 0;

toplevel DUT (clk, packet, core_X, core_Y, core_availability_signals_out, core_availability_signals_in, core_inputs_2_bits);

localparam CLK_PERIOD = 10;
always #(CLK_PERIOD/2) clk=~clk;

initial begin
    $dumpfile("tb/tb.vcd");

	for (k = 0; k < `Y; k = k + 1)
	begin
		for (p = 0; p < `X; p = p + 1)
		begin
			memory[k][p][0] = 0;
			memory[k][p][1] = 0;
		end
	end
	
	for (k = 0; k < `Y; k = k + 1)
	begin
		for (p = 0; p < `X; p = p + 1)
		begin
			core_availability_signals_out[k][p] = 1;
		end
	end
	
	for (i = 0; i < `Y; i = i + 1)
	begin
		for (j = 0; j < `X; j = j + 1)
		begin
			for (k = 0; k < `Y; k = k + 1)
			begin
				for (p = 0; p < `X; p = p + 1)
				begin
					@(posedge clk)
					begin
						core_Y <= i;
						core_X <= j;
						packet <= {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b0}}};
					end
					@(posedge clk)
					begin
						packet <= {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b1}}};
					end
					@(posedge clk)
					begin
						packet <= 0;
					end
				end
			end
			#1000;
			for (k = 0; k < `Y; k = k + 1)
			begin
				for (p = 0; p < `X; p = p + 1)
				begin
					assert (memory[k][p][0] == {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b0}}}) else $error("Packet %b not to be seen", {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b0}}});
					assert (memory[k][p][1] == {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b1}}}) else $error("Packet %b not to be seen", {1'b1, p[`CS-1:0], k[`CS-1:0], {(`PL-1-`CS-`CS){1'b1}}});
					memory[k][p][0] = 0;
					memory[k][p][1] = 0;
				end
			end
		end
	end
	#200;
    $finish;

end

always @(*)
begin

	for (t = 0; t < `Y; t = t + 1)
	begin
		for (m = 0; m < `X; m = m + 1)
		begin
			if (DUT.core_inputs[t][m] != 0)
				if (DUT.core_inputs[t][m][`PL-1] == 0)
					memory[t][m][0] = DUT.core_inputs[t][m];
				else
					memory[t][m][1] = DUT.core_inputs[t][m];
		end
	end

end

endmodule
